/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.pi;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.concurrent.ThreadLocalRandom;

import org.apache.log4j.Logger;

import com.moneyhouse.tmor.serial.pi.dto.DSSensorResultDTO;

/**
 * READ OUT DS One wire temp sensor from basisstation PI
 * 
 * @author Frederic
 * @since 13.07.2015
 * ADDED A SEARCH FUNCTION CALLED BEFOReREADING THE THE SENSOR- AS WE GET NULL POINTER EXCEPTIONS
 */
public class DSSensor {
	private final static Logger logger = Logger.getLogger(DSSensor.class); 	
	private boolean isUnixFileSystem = false;
	
	/**
	 * Default constructor
	 */
	public DSSensor(){
		if("/".equals(System.getProperty("file.separator"))){
			this.isUnixFileSystem = true;
			logger.info("WE ARE RUNNING ON THE TARGET LINUX SYSTEM");
		} else if("\\".equals(System.getProperty("file.separator"))){
			this.isUnixFileSystem = false;
			logger.warn("WARN: WE ARE NOT RUNNING ON THE TARGET LINUX SYSTEM - THERE ARE NO MEANINGFUL RESULTS");
			
		} else {
			throw new RuntimeException("ERROR: EXPCETED TO BE UNIX OR NON UNIX FILESYSTEM - BUT WE GOT ["+System.getProperty("file.separator")+"]");
		}
	}
	
	
		/**
		 * devices are in folder  /sys/bus/w1/devices
		 * folder w1 does not exist when /boot/config.txt does not contain dtoverlay=w1-gpio
		 * it only works on pi target plattform
		 */
	public ArrayList<String> readSensorList(){
		logger.info("-----> DSSensor.readSensorList()");
		ArrayList<String> retVal = new ArrayList<String>();
		if(isUnixFileSystem){
			readSensorListUnix(retVal);
		} 
		else 
		{
			readSensorListWindows(retVal);
		}
		String printoutSensorlist = "";
		for(int h = 0 ; h < retVal.size(); h++ ){
			printoutSensorlist += "\t" + retVal.get(h);
		}
		logger.info("<----- DSSensor.readSensorList() SIZE= ["+retVal.size()+"]"+printoutSensorlist);
		return retVal; 
	}


		/**
		 * IT DOES NOT WORK UNDEr WINDOWS - SO WE RETURN DUMMY SENSORS
		 * @param retVal
		 */
		private void readSensorListWindows(ArrayList<String> retVal) {
			logger.warn("WARN: WE ARE NOT RUNNING ON THE TARGET LINUX SYSTEM - THERE ARE NO MEANINGFUL RESULTS - ADDING ARTIFICAL 28-000004475b0d AND 28-00000460d5b1");
			
			retVal.add("28-000004475b0d");
			retVal.add("28-00000460d5b1");
			
			
			retVal.add("28-00000460dnew");
		}


		private void readSensorListUnix(ArrayList<String> retVal) {
			if(doesOneWireDirectoryExist()){
				// YES IT EXISTS
				if(doesOneWireDevicesExist()){
					// YES IT EXISTS - READ OUT FILES
					File devices = new File("/sys/bus/w1/devices");
					File[] fileArray = devices.listFiles();
					if(fileArray.length == 0){
						// NO DEVICES CONNECTED
						logger.info("NO TEMPERATURE SENSOR CONNCETED TO THE BASIS STATION");
					}
					else 
					{
						// SOME DEVICE CONNECTED 
						for(int i = 0 ; i < fileArray.length; i++){
							File afile = fileArray[i];
							if(afile.getName().equalsIgnoreCase("w1_bus_master1")){
								// THIS ONE IS NOT A DEVICE - DO NOTHING
								
							} else if(afile.getName().startsWith("28-")) {
								// YES ITS A ONE WIRE TEMPERATURE SENSOR
								retVal.add(afile.getName());
							} else {
								logger.error("ERROR: PATH /sys/bus/w1/devices CONTAINS FILE ["+afile.getName()+"] THAT IS NOT KNOWN - IS IT A DALLAS ONE WIRE TEMPERATURE SENSOR DS18B20");
							}
						}
					}
				} 
				else 
				{
					// NO - is it a bug?
					logger.warn("WARNING: IT SEEMS THE PATH /sys/bus/w1/devices DOES NOT EXIST - CHECK /boot/config.txt FILE");
				}
			} 
			else 
			{
				logger.warn("WARNING: IT SEEMS THE PATH /sys/bus/w1 DOES NOT EXIST - ONE WIRE CONFIGURATION ON BASIS STATION IS NOT CORRECT - CHECK /boot/config.txt FILE");
			}
		}
	

	
	public DSSensorResultDTO readSensorValueWithID(String sid){
		DSSensorResultDTO retVal = new DSSensorResultDTO();
		logger.info("-----> DSSensor.readSensorValueWithID("+sid+")");
		
		if(isUnixFileSystem){
			readSensorValueWithIDUnix(sid, retVal);
		} else {
			readSensorValueWithIDWindows(retVal);

		}
		logger.info("<----- DSSensor.readSensorValueWithID("+sid+") retVal="+retVal);
		return retVal;
	}


	/** RETURN DUMMY VALUES AROUND 8 DEGREES
	 * @param retVal
	 */
	private void readSensorValueWithIDWindows(DSSensorResultDTO retVal) {
		// NOT RUNNING UNDER LINUX
		logger.warn("WARN: WE ARE NOT RUNNING ON THE TARGET LINUX SYSTEM - THERE ARE NO MEANINGFUL RESULTS");
		logger.warn("WARN: WE RETURN 12.3");
		BigDecimal calculated = new BigDecimal("8.0");
		
		// just to give it some randomness 
		int rndnubr = ThreadLocalRandom.current().nextInt(-1, 1 + 1);
		calculated = calculated.add(new BigDecimal(rndnubr), MathContext.DECIMAL32);
		
		
		retVal.setReadingValueBD(calculated);
		retVal.setReadingValueStr(calculated.toString());
		retVal.setReadingValueValid(true);
		logger.warn("WARN: WE RETURN 12.3 OR "+ calculated.toString());
	}


	private void readSensorValueWithIDUnix(String sid, DSSensorResultDTO retVal) {
		// check whether the sensor is connected to the base station
		isSensorConnected(sid);
		
		
		// logger.debug("1");
		File theDevice = null;
		for(int tries = 0 ;  tries < 5; tries++){
			// logger.debug("2 try to open - attempt ["+tries+"]");
			theDevice = openFile(sid);
			if(theDevice != null ){
				break;
			} else {
				try{
					Thread.sleep(1000);
				} catch(Exception e){
					logger.error("ERROR: WHILE WAITING FOR FILE LOCATION FOR ["+sid+"] NOT TO RETURN NULL WE CATCHED THE FOLLOWING EXCPETION ", e);
				}
			}
		}
		// logger.debug("3 theDevice = "+ theDevice);

		if(theDevice == null){
			// STILL NULL
			logger.warn("WARN: THE FILE LOCATION FOR ["+sid+"] RETURNS NULL - WE DELIVER A RESULT WHICH IS NOT VALID!!");
			
			retVal.setReadingValueValid(false);
		} 
		else 
		{
			// logger.debug("4");
			File[] fileArray = theDevice.listFiles();
			if(fileArray == null){
				for(int i = 0 ; i < 10; i++){
					logger.fatal("FATAL: HERE IS SOMe PROBLEM WITH READINg THE OW DEVICES ON BASE STATION DIR: /sys/bus/w1/devices - WE WANTED TO READ ["+sid+"] BUT THE FILEARRAY IS NULL - REBOOT THE WHOLE BASE STATION AND CHECK AGAIN!! ");
					
				}
			}
			for(int i = 0 ; i < fileArray.length; i++){
				File file = fileArray[i];
				if(file.getName().equalsIgnoreCase("name")){
					// READ NAME FILE
					String content = readFile("/sys/bus/w1/devices/"+sid+"/name").trim();
					logger.info("CONTENT OF FILE name = ["+content+"]");
					if(content.trim().equals(sid)){
						// YES WE ARE IN tHE RIGHT DIRECTORY - READINg THE RIGHT DEVICE
						retVal.setSensorID(content.trim());
					} else {
						// SOMETHING WENT WRONG - WE ARE NOT IN tHE RIGHT DIRECTORY 
						logger.error("ERROR: WE WANT TO READ sid = ["+sid+"] BUT READ name FILE CONTENT ["+content.trim()+"]");
						retVal.setSensorID("ERROR: WE WANT TO READ sid = ["+sid+"] BUT READ name FILE CONTENT ["+content.trim()+"]");
					}
				}
				// logger.debug("5");

				if(file.getName().equalsIgnoreCase("w1_slave")){
					// READ TEMPERATURE - multiple times 
					for(int ii = 0; ii < 10; ii++){
						String content = readFile("/sys/bus/w1/devices/"+sid+"/w1_slave");
						// logger.debug("READ ATTEMPT NR ["+ii+"] CONTENT OF FILE w1_slave = ["+content+"]");
						if(content.contains("YES") && content.contains("t=") ){
							retVal.setReadingValueValid(true);
							
							StringTokenizer st = new StringTokenizer(content, "t=");
//							StringTokenizer st2 = new StringTokenizer(content, "t=");
//							int toki = 0;
//							while(st2.hasMoreTokens()){
//								toki++;
//								// logger.debug("TOKEN  ["+toki+"] \t "+ st2.nextToken());
//							}
							
							
							
							int tokenCount = st.countTokens();
//							// logger.debug("TOKENCOUNT =  ["+tokenCount+"]");

							
							if(tokenCount == 3){
								// OK THIS IS EXPECTED
								String temperature = "";
								try{
									st.nextToken();
									st.nextToken();
									temperature = st.nextToken();
//									// logger.debug("TEMPERATURE = ["+temperature+"]");
								
									BigDecimal raw = new BigDecimal(temperature.trim());
									BigDecimal calculated = raw.divide(new BigDecimal(1000), MathContext.DECIMAL32);
									retVal.setReadingValueBD(calculated);
									retVal.setReadingValueStr(calculated.toString());
								} catch (NumberFormatException nfe){
									logger.error("ERROR: CONVERTING TEMPERATURE ["+temperature+"] TO NUMBER");
								} catch (Exception oe){
									
									logger.error("ERROR: OTHER ERROR OCCURED WHILE READING w1_slave FILE", oe);
								}
							} 
							else 
							{
								logger.error("ERROR: READING FILE [/sys/bus/w1/devices/"+sid+"/w1_slave] EXPECTED 2 TOKENS BUT HAVE ["+tokenCount+"] - CHECK CONTENT ["+content+"]");
							}
							break;
						}
						try{
							// READING NOT SUCCESSFUL - SO WAIT A LITTLE AND REREAD
							Thread.sleep(1000);
						} catch(InterruptedException ie){
							logger.error("ERROR: SLEEP INTERRUPPTED" , ie);
						}
					}
				}
				// logger.debug("6");
			}
		}
	}


	/**QUERIES THROUGH THE LIST OF SENSORS AND CHECKS WHETHER THE sid CAN BE FOUND IN IT
	 * @param sid
	 */
	private boolean isSensorConnected(String sid) {
		logger.info("-----> DSSensor.isSensorConnected("+sid+")");
		boolean retval = false;
		ArrayList<String> connected = readSensorList();
		Iterator<String> connected_it = connected.iterator();
		while(connected_it.hasNext()){
			String conn = connected_it.next();
			if(conn.equals(sid)){
				// found it in list - Ok
				retval = true;
				break;
			} 
		}
		logger.info("<----- DSSensor.isSensorConnected("+sid+") retVal: "+retval);
		return retval;
	}


	private File openFile(String sid) {
		File theDevice;
		try{
			theDevice = new File("/sys/bus/w1/devices/"+sid);
			
		} catch(NullPointerException ex){
			logger.error("ERROR: READING THE DEVICE= ["+sid+"] ON PATH ["+"/sys/bus/w1/devices/"+"] RETURNS NULL");
			theDevice = null;
			
		}
		return theDevice;
	}
	
	private boolean doesOneWireDirectoryExist(){
		// logger.debug("-----> DSSensor.doesOneWireDirectoryExist()");
		boolean retVal = false;
		
		File w1 = new File("/sys/bus/w1");
		retVal = w1.exists();

		// logger.debug("<----- DSSensor.doesOneWireDirectoryExist()");
		return retVal; 
	}

	private boolean doesOneWireDevicesExist(){
		// logger.debug("-----> DSSensor.doesOneWireDevicesExist()");
		boolean retVal = false;
		
		File devices = new File("/sys/bus/w1/devices");
		retVal = devices.exists();
		
		// logger.debug("<----- DSSensor.doesOneWireDevicesExist()");
		return retVal; 
	}
	
	
	private String readFile(String path) 
	{
		String retVal = "";
		byte[] encoded;
		try {
			encoded = Files.readAllBytes(Paths.get(path));
			retVal = new String(encoded);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR: READING FILE ["+Paths.get(path)+"]", e);
		}
		return retVal;
	}
}
